#include <dolfin.h>
#include <dolfin/timing.h>
#include <dolfin/MeshFunction.h>
#include <dolfin/Assembler.h>
#include "NSEMomentum2D.h"

using namespace dolfin;

real bmarg = 1.0e-3 + DOLFIN_EPS;
//real bmarg = DOLFIN_EPS;

// 2D data:
real xmin = 0.0;
real xmax = 1.0;
real ymin = 0.0;
real ymax = 1.0;
real zmin = 0.0;
real zmax = 0.0;

// Sub domain for inflow
class InflowTopBottomBoundary2D : public SubDomain
{
public:
  bool inside(const real* p, bool on_boundary) const
  {
    return on_boundary && p[0] < xmax - bmarg && 
      p[0] < bmarg || p[1] < ymin + bmarg || p[1] > ymax - bmarg;
  }
};

// Boundary condition for momentum equation 
class BC_Momentum_2D : public Function
{
public:
  BC_Momentum_2D(Mesh& mesh) : Function(mesh) {}
  void eval(real* values, const real* x) const
  {
    for(int i = 0; i < 2; i++)
      {
	values[i] = 0.0;
      }
  }
};

int main()
{
  UnitSquare mesh(100, 100);

  InflowTopBottomBoundary2D iboundary;

  BC_Momentum_2D bcf_mom(mesh);

  Function f(mesh, 0.0);

  DirichletBC bc_mom0(bcf_mom, mesh, iboundary);

  //Array <BoundaryCondition*> bcs(&bc_mom0, &bc_mom1);
  Array <BoundaryCondition*> bcs;
  bcs.push_back(&bc_mom0);


  real foo = 1.0;

  Function u(mesh, foo);   // velocity from previous time step 
  Function u0(mesh, foo);  // velocity from previous time step 
  Function uc(mesh, foo);  // velocity linearized convection 
  Function um(mesh, foo);  // cell mean velocity linearized convection 
  Function p(mesh, foo);   // current pressure
  Function delta1(mesh, foo), delta2(mesh, foo);
  
  Function fk(mesh, foo);
  Function fnu(mesh, foo);

  Form* amom = 0;
  Form* Lmom = 0;

  amom = new NSEMomentum2DBilinearForm(um,delta1,delta2,fk,fnu);
  Lmom = new NSEMomentum2DLinearForm(um,u0,f,p,delta1,delta2,fk,fnu);

  // Manual implementation

//   Matrix Amom;
//   Vector bmom;

//   Assembler assembler(mesh);
//   assembler.assemble(Amom, *amom);
//   assembler.assemble(bmom, *Lmom);

//   for (dolfin::uint i = 0; i < bcs.size(); i++)
//     bcs[i]->apply(Amom, bmom, *amom);

  // Automatic implementation

  LinearPDE pde(*amom, *Lmom, mesh, bcs);
  
  Function U;
  pde.set("PDE linear solver", "direct");
  pde.solve(U);

  return 0;
}
